'''
python -m unittest tests/grassat_model_test.py
'''

import unittest
from unittest.mock import patch
from grassat.database.meteo import MeteoDatabase
from grassat.model.field_data import *
from pathlib import Path
import json
import unittest
from fastapi.testclient import TestClient
from grassat.main import app   # Ensure this is your FastAPI app's module


class TestGrassatModel(unittest.TestCase):
    
    @patch("grassat.database.base.Elasticsearch")  # Mock the Elasticsearch client class
    def setUp(self, MockElasticsearch):
        # Initialize the FastAPI test client
        self.client = TestClient(app)
        
         # Mock Elasticsearch instance
        self.mock_es = MockElasticsearch.return_value
        
        # Initialize MeteoDatabase after mocking Elasticsearch
        self.meteo_db = MeteoDatabase()
        

    def test_run_model_with_empty_irrigations_and_fertilisations(self):
        # Load mock request and expected response from a JSON file as a dictionary
        with open(Path(__file__).parent.parent / "tests" / "fixtures" / "grassatRequest1.json", 'r') as file:
            self.mock_request_body = json.load(file)
            
        with open(Path(__file__).parent.parent / "tests" / "fixtures" / "expectedResponse1.json", 'r') as file:
            self.expectedResponse = json.load(file)
            
        # Send POST request to the endpoint
        response = self.client.post("/model", json=self.mock_request_body)

        # Validate the response
        self.assertEqual(response.status_code, 200)

        # Assert that the response matches the expected response
        response_json = response.json()
        self.assertEqual(response_json, self.expectedResponse)
        
    def test_run_model_with_irrigations_and_fertilisations(self):
        # Load mock request and expected response from a JSON file as a dictionary
        with open(Path(__file__).parent.parent / "tests" / "fixtures" / "grassatRequest2.json", 'r') as file:
            self.mock_request_body = json.load(file)
            
        with open(Path(__file__).parent.parent / "tests" / "fixtures" / "expectedResponse2.json", 'r') as file:
            self.expectedResponse = json.load(file)
        
        # Send POST request to the endpoint
        response = self.client.post("/model", json=self.mock_request_body)

        # Validate the response
        self.assertEqual(response.status_code, 200)
        
        # Assert that the response matches the expected response
        response_json = response.json()
        self.assertEqual(response_json, self.expectedResponse)
        
    def test_run_model_with_current_year_start_date(self):
        # Load mock request and expected response from a JSON file as a dictionary
        with open(Path(__file__).parent.parent / "tests" / "fixtures" / "grassatRequest3.json", 'r') as file:
            self.mock_request_body = json.load(file)
           
        # Send POST request to the endpoint
        response = self.client.post("/model", json=self.mock_request_body)

        # Validate the response
        self.assertEqual(response.status_code, 200)
        
        #TODO: This test should be updated when current season issue is fixed. 
        # We get 200 in response here, because in model class startDate and endDate is replaced temporarily.
        # Normally, when startDate is begin of the current year, model throws error. 
        # Becasue it expects 365 days of meteo data while it doesn't get it.
        
        
    def test_no_meteo_data(self):
        """
        Test case where no meteo data is found for the geometry
        """
        # Load mock request from a JSON file as a dictionary
        with open(Path(__file__).parent.parent / "tests" / "fixtures" / "grassatRequest1.json", 'r') as file:
            self.mock_request_body = json.load(file)
                  
        field_data: FieldData = FieldData.from_request(self.mock_request_body['grassatRequest'])
        
        # Elasticsearch search query result - no hits
        self.mock_es.search.return_value = {"hits": {"hits": []}}

        # Test whether exception is raised when no meteo data is found
        with self.assertRaises(Exception) as context:
            self.meteo_db.get_meteo(field_data)

        # Check that the exception message is correct
        self.assertTrue(
            "Could not find meteo information for geometry" in str(context.exception)
        )



if __name__ == '__main__':
    unittest.main()
